package cz.vutbr.fit.gja;

import java.util.ArrayList;
import java.util.List;

import org.springframework.stereotype.Controller;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
public class HomeController {

  private static List<State> states;

  /**
   * Simply selects the home view to render by returning its name.
   *
   * @return
   */
  @RequestMapping(value = "/", method = RequestMethod.GET)
  public String start() {
    return "home";
  }

  /**
   * Retrieve a list of states. Accepts a GET request for JSON
   *
   * @return A JSON array of states
   */
  @RequestMapping(value = "states", method = RequestMethod.GET, produces = "application/json")
  public @ResponseBody
  List<State> fetchStatesJson() {
    return getStates();
  }

  /**
   * Retrieve a list of states. Accepts a GET request for XML
   *
   * @return An XML array of states
   */
  @RequestMapping(value = "states", method = RequestMethod.GET, produces = "application/xml")
  public @ResponseBody
  StateList fetchStatesXml() {
    List<State> statesL = getStates();
    StateList stateList = new StateList(statesL);
    return stateList;
  }

  /**
   * Retrieve a single state. Accepts a GET request for JSON with a parameter
   * for the state abbreviation
   *
   * @param abbreviation contains the state abbreviation to use when finding the
   * corresponding state
   * @return A JSON state
   */
  @RequestMapping(value = "state/{abbreviation}", method = RequestMethod.GET, produces = "application/json")
  public @ResponseBody
  State fetchStateJson(@PathVariable String abbreviation) {
    return getStateByAbbreviation(abbreviation);
  }

  /**
   * Retrieve a single state. Accepts a GET request for XML with a parameter for
   * the state abbreviation
   *
   * @param abbreviation contains the state abbreviation to use when finding the
   * corresponding state
   * @return An XML state
   */
  @RequestMapping(value = "state/{abbreviation}", method = RequestMethod.GET, produces = "application/xml")
  public @ResponseBody
  State fetchStateXml(@PathVariable String abbreviation) {
    return getStateByAbbreviation(abbreviation);
  }

  /**
   * Accepts a POST request with a plain text message parameter
   *
   * @param message contains the body of the POST request
   * @return a string with the result of the POST
   */
  @RequestMapping(value = "sendmessage", method = RequestMethod.POST, consumes = "text/plain")
  public @ResponseBody
  String sendMessage(@RequestBody String message) {
    return "String message received! Your message: " + message;
  }

  /**
   * Accepts a POST request with a JSON message parameter
   *
   * @param message serialized Message object
   * @return a string with the result of the POST
   */
  @RequestMapping(value = "sendmessage", method = RequestMethod.POST, consumes = "application/json")
  public @ResponseBody
  String sendMessageJson(@RequestBody Message message) {
    return "JSON message received! Your message: " + message.toString();
  }

  /**
   * Accepts a POST request with an XML message parameter
   *
   * @param message serialized Message object
   * @return a string with the result of the POST
   */
  @RequestMapping(value = "sendmessage", method = RequestMethod.POST, consumes = "application/xml")
  public @ResponseBody
  String sendMessageXml(@RequestBody Message message) {
    return "XML message received! Your message: " + message.toString();
  }

  /**
   * Accepts a POST request with an Map message parameter, and creates a new
   * Message object from the Map parameters.
   *
   * @param map serialized LinkedMultiValueMap<String, String> object
   * @return a string with the result of the POST
   */
  @RequestMapping(value = "sendmessagemap", method = RequestMethod.POST)
  public @ResponseBody
  String sendMessageMap(@RequestBody LinkedMultiValueMap<String, String> map) {
    Message message = new Message();

    try {
      message.setId(Integer.parseInt(map.getFirst("id")));
    } catch (NumberFormatException e) {
      message.setId(0);
    }

    message.setSubject(map.getFirst("subject"));
    message.setText(map.getFirst("text"));

    return "Map message received! Your message: " + message.toString();
  }

  // helper methods
  private State getStateByAbbreviation(String abbreviation) {
    List<State> statesList = getStates();

    for (State state : statesList) {
      if (state.getAbbreviation().compareToIgnoreCase(abbreviation) == 0) {
        return state;
      }
    }

    return null;
  }

  /**
   * @return a List of states
   */
  private List<State> getStates() {
    if (states == null) {
      states = new ArrayList<State>();
      states.add(new State("ALABAMA", "AL"));
      states.add(new State("ALASKA", "AK"));
      states.add(new State("ARIZONA", "AZ"));
      states.add(new State("ARKANSAS", "AR"));
      states.add(new State("CALIFORNIA", "CA"));
      states.add(new State("COLORADO", "CO"));
      states.add(new State("CONNECTICUT", "CT"));
      states.add(new State("DELAWARE", "DE"));
      states.add(new State("FLORIDA", "FL"));
      states.add(new State("GEORGIA", "GA"));
      states.add(new State("HAWAII", "HI"));
      states.add(new State("IDAHO", "ID"));
      states.add(new State("ILLINOIS", "IL"));
      states.add(new State("INDIANA", "IN"));
      states.add(new State("IOWA", "IA"));
      states.add(new State("KANSAS", "KS"));
      states.add(new State("KENTUCKY", "KY"));
      states.add(new State("LOUISIANA", "LA"));
      states.add(new State("MAINE", "ME"));
      states.add(new State("MARYLAND", "MD"));
      states.add(new State("MASSACHUSETTS", "MA"));
      states.add(new State("MICHIGAN", "MI"));
      states.add(new State("MINNESOTA", "MN"));
      states.add(new State("MISSISSIPPI", "MS"));
      states.add(new State("MISSOURI", "MO"));
      states.add(new State("MONTANA", "MT"));
      states.add(new State("NEBRASKA", "NE"));
      states.add(new State("NEVADA", "NV"));
      states.add(new State("NEW HAMPSHIRE", "NH"));
      states.add(new State("NEW JERSEY", "NJ"));
      states.add(new State("NEW MEXICO", "NM"));
      states.add(new State("NEW YORK", "NY"));
      states.add(new State("NORTH CAROLINA", "NC"));
      states.add(new State("NORTH DAKOTA", "ND"));
      states.add(new State("OHIO", "OH"));
      states.add(new State("OKLAHOMA", "OK"));
      states.add(new State("OREGON", "OR"));
      states.add(new State("PENNSYLVANIA", "PA"));
      states.add(new State("RHODE ISLAND", "RI"));
      states.add(new State("SOUTH CAROLINA", "SC"));
      states.add(new State("SOUTH DAKOTA", "SD"));
      states.add(new State("TENNESSEE", "TN"));
      states.add(new State("TEXAS", "TX"));
      states.add(new State("UTAH", "UT"));
      states.add(new State("VERMONT", "VT"));
      states.add(new State("VIRGINIA", "VA"));
      states.add(new State("WASHINGTON", "WA"));
      states.add(new State("WEST VIRGINIA", "WV"));
      states.add(new State("WISCONSIN", "WI"));
      states.add(new State("WYOMING", "WY"));
    }

    return states;
  }

}
